@@ -143,6 +143,10 @@ class FixIncludesBase(unittest.TestCase):
          '==== targets',
          '//foo/bar:a'])
 
+    # Stub out stdout
+    self.stdout_stub = StringIO()
+    fix_includes.sys.stdout = self.stdout_stub
+
   def SetFindClientsOfFilesOutput(self, output_lines):
     self.fake_get_command_output_lines.SetCommandOutputLines(
         self.find_clients_of_files_command, output_lines)
@@ -264,6 +268,39 @@ The full include-list for simple:
     self.RegisterFileContents({'simple': infile})
     self.ProcessAndTest(iwyu_output, expected_num_modified_files=1)
 
+  def testUnifiedDiffOutput(self):
+    """Test the unified diff output generated by dry runs."""
+    infile = """
+#include <notused.h>
+
+int main() { return 0; }
+"""
+    iwyu_output = """\
+unified_diff.cc should add these lines:
+
+unified_diff.cc should remove these lines:
+- #include <notused.h>  // lines 2-2
+
+The full include-list for unified_diff.cc:
+---
+"""
+    diff_expect = """\
+>>> Fixing #includes in 'unified_diff.cc'
+@@ -1,4 +1,2 @@
+-
+-#include <notused.h>
+
+ int main() { return 0; }
+IWYU edited 0 files on your behalf.
+
+"""
+
+    self.flags.dry_run = True
+    self.RegisterFileContents({'unified_diff.cc': infile})
+    self.ProcessAndTest(iwyu_output, unedited_files=['unified_diff.cc'])
+
+    self.assertEqual(self.stdout_stub.getvalue(), diff_expect)
+
   def testNodiffOutput(self):
     """Tests handling of the '(<file> has correct #includes)' iwyu output."""
     infile = """\
