@@ -22,6 +22,7 @@
 #include "iwyu_string_util.h"
 #include "iwyu_verrs.h"
 #include "port.h"  // for CHECK_
+#include "llvm/ADT/ArrayRef.h"
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/raw_ostream.h"
 #include "clang/AST/ASTContext.h"
@@ -73,11 +74,8 @@ using clang::ExplicitCastExpr;
 using clang::Expr;
 using clang::ExprWithCleanups;
 using clang::FileEntry;
-using clang::FriendDecl;
-using clang::FriendTemplateDecl;
 using clang::FullSourceLoc;
 using clang::FunctionDecl;
-using clang::FunctionProtoType;
 using clang::FunctionType;
 using clang::ImplicitCastExpr;
 using clang::InjectedClassNameType;
@@ -86,7 +84,6 @@ using clang::MemberExpr;
 using clang::MemberPointerType;
 using clang::NamedDecl;
 using clang::NestedNameSpecifier;
-using clang::NestedNameSpecifierLoc;
 using clang::ObjCObjectType;
 using clang::OverloadExpr;
 using clang::PointerType;
@@ -96,7 +93,6 @@ using clang::RecordDecl;
 using clang::RecordType;
 using clang::RecursiveASTVisitor;
 using clang::SourceLocation;
-using clang::SourceManager;
 using clang::SourceRange;
 using clang::Stmt;
 using clang::SubstTemplateTypeParmType;
@@ -120,8 +116,10 @@ using clang::UnaryOperator;
 using clang::UsingDirectiveDecl;
 using clang::ValueDecl;
 using clang::VarDecl;
+using llvm::ArrayRef;
 using llvm::PointerUnion;
 using llvm::cast;
+using llvm::dyn_cast;
 using llvm::dyn_cast_or_null;
 using llvm::errs;
 using llvm::isa;
@@ -158,7 +156,7 @@ void DumpASTNode(llvm::raw_ostream& ostream, const ASTNode* node) {
   }
 }
 
-}
+}  // anonymous namespace
 
 //------------------------------------------------------------
 // ASTNode and associated utilities.
@@ -169,7 +167,7 @@ SourceLocation ASTNode::GetLocation() const {
     return retval;
 
   // OK, let's ask a parent node.
-  for (const ASTNode* node = parent_; node != NULL; node = node->parent_) {
+  for (const ASTNode* node = parent_; node != nullptr; node = node->parent_) {
     if (node->FillLocationIfKnown(&retval))
       break;
   }
@@ -221,23 +219,19 @@ bool ASTNode::FillLocationIfKnown(SourceLocation* loc) const {
 // --- Utilities for ASTNode.
 
 bool IsElaborationNode(const ASTNode* ast_node) {
-  if (ast_node == NULL)
+  if (ast_node == nullptr)
     return false;
   const ElaboratedType* elaborated_type = ast_node->GetAs<ElaboratedType>();
   return elaborated_type && elaborated_type->getKeyword() != clang::ETK_None;
 }
 
-bool IsNamespaceQualifiedNode(const ASTNode* ast_node) {
-  if (ast_node == NULL)
+bool IsQualifiedNameNode(const ASTNode* ast_node) {
+  if (ast_node == nullptr)
     return false;
   const ElaboratedType* elaborated_type = ast_node->GetAs<ElaboratedType>();
-  if (elaborated_type == NULL)
+  if (elaborated_type == nullptr)
     return false;
-  const NestedNameSpecifier* qualifier = elaborated_type->getQualifier();
-  if (qualifier == NULL)
-    return false;
-  return (qualifier->getKind() == NestedNameSpecifier::Global ||
-          qualifier->getKind() == NestedNameSpecifier::Namespace);
+  return elaborated_type->getQualifier() != nullptr;
 }
 
 bool IsNodeInsideCXXMethodBody(const ASTNode* ast_node) {
@@ -246,7 +240,7 @@ bool IsNodeInsideCXXMethodBody(const ASTNode* ast_node) {
   // triggered when we see implicit destruction of member vars.
   if (ast_node && ast_node->IsA<CXXDestructorDecl>())
     return true;
-  for (; ast_node != NULL; ast_node = ast_node->parent()) {
+  for (; ast_node != nullptr; ast_node = ast_node->parent()) {
     // If we're a constructor, check if we're part of the
     // initializers, which also count as 'the body' of the method.
     if (const CXXConstructorDecl* ctor =
@@ -314,11 +308,11 @@ template<typename T>
 NestedNameSpecifier* TryGetQualifier(const ASTNode* ast_node) {
   if (ast_node->IsA<T>())
     return ast_node->GetAs<T>()->getQualifier();
-  return NULL;
+  return nullptr;
 }
 
 const NestedNameSpecifier* GetQualifier(const ASTNode* ast_node) {
-  const NestedNameSpecifier* nns = NULL;
+  const NestedNameSpecifier* nns = nullptr;
   if (ast_node->IsA<TemplateName>()) {
     const TemplateName* tn = ast_node->GetAs<TemplateName>();
     if (const DependentTemplateName* dtn
@@ -364,7 +358,7 @@ bool IsMemberOfATypedef(const ASTNode* ast_node) {
     if (nns && ast_node->IsA<TypedefType>()) {
       nns = nns->getPrefix();
     } else if (!nns) {
-      // nns will be non-NULL when processing 'a' in MyTypedef::a::b
+      // nns will be non-nullptr when processing 'a' in MyTypedef::a::b
       // But typically, such as processing 'a' in MyTypedef::a or 'b' in
       // MyTypedef::a::b, the parent will be an ElaboratedType.
       if (const ElaboratedType* elab_type =
@@ -381,14 +375,13 @@ bool IsMemberOfATypedef(const ASTNode* ast_node) {
 }
 
 const DeclContext* GetDeclContext(const ASTNode* ast_node) {
-  for (; ast_node != NULL; ast_node = ast_node->parent()) {
+  for (; ast_node != nullptr; ast_node = ast_node->parent()) {
     if (ast_node->IsA<Decl>())
       return ast_node->GetAs<Decl>()->getDeclContext();
   }
-  return NULL;
+  return nullptr;
 }
 
-
 //------------------------------------------------------------
 // Helper functions for working with raw Clang AST nodes.
 
@@ -454,7 +447,7 @@ string PrintableTemplateArgument(const TemplateArgument& arg) {
   std::string buffer;
   raw_string_ostream ostream(buffer);
   TemplateSpecializationType::PrintTemplateArgumentList(
-      ostream, &arg, 1, DefaultPrintPolicy());
+      ostream, ArrayRef<TemplateArgument>(arg), DefaultPrintPolicy());
   return ostream.str();
 }
 
@@ -462,7 +455,7 @@ string PrintableTemplateArgumentLoc(const TemplateArgumentLoc& arg) {
   std::string buffer;
   raw_string_ostream ostream(buffer);
   TemplateSpecializationType::PrintTemplateArgumentList(
-      ostream, &arg, 1, DefaultPrintPolicy());
+      ostream, ArrayRef<TemplateArgumentLoc>(arg), DefaultPrintPolicy());
   return ostream.str();
 }
 
@@ -493,11 +486,11 @@ string GetWrittenQualifiedNameAsString(const NamedDecl* named_decl) {
 
 // If the TemplateArgument is a type (and not an expression such as
 // 'true', or a template such as 'vector', etc), return it.  Otherwise
-// return NULL.
+// return nullptr.
 static const Type* GetTemplateArgAsType(const TemplateArgument& tpl_arg) {
   if (tpl_arg.getKind() == TemplateArgument::Type)
     return tpl_arg.getAsType().getTypePtr();
-  return NULL;
+  return nullptr;
 }
 
 // These utilities figure out the template arguments that are
@@ -516,7 +509,7 @@ static const Type* GetTemplateArgAsType(const TemplateArgument& tpl_arg) {
 // VarDecl 'vector<int(*)(const MyClass&)> x', it would return
 // (vector<int(*)(const MyClass&)>, int(*)(const MyClass&),
 // int(const MyClass&), int, const MyClass&, MyClass).  Note that
-// this function only returns types-as-typed, so it does *not* return
+// this function only returns types-as-written, so it does *not* return
 // alloc<int(*)(const MyClass&)>, even though it's part of vector.
 class TypeEnumerator : public RecursiveASTVisitor<TypeEnumerator> {
  public:
@@ -559,7 +552,7 @@ set<const Type*> GetComponentsOfType(const Type* type) {
 // --- Utilities for Decl.
 
 bool IsTemplatizedFunctionDecl(const FunctionDecl* decl) {
-  return decl && decl->getTemplateSpecializationArgs() != NULL;
+  return decl && decl->getTemplateSpecializationArgs() != nullptr;
 }
 
 bool HasImplicitConversionCtor(const CXXRecordDecl* cxx_class) {
@@ -607,7 +600,7 @@ const RecordDecl* GetDefinitionForClass(const Decl* decl) {
       return record_dfn;
     }
     // If we're a templated class that was never instantiated (because
-    // we were never "used"), then getDefinition() will return NULL.
+    // we were never "used"), then getDefinition() will return nullptr.
     if (const ClassTemplateSpecializationDecl* spec_decl = DynCastFrom(decl)) {
       PointerUnion<ClassTemplateDecl*,
           ClassTemplatePartialSpecializationDecl*>
@@ -629,7 +622,7 @@ const RecordDecl* GetDefinitionForClass(const Decl* decl) {
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 SourceRange GetSourceRangeOfClassDecl(const Decl* decl) {
@@ -674,19 +667,18 @@ set<FunctionDecl*> GetLateParsedFunctionDecls(TranslationUnitDecl* decl) {
 // contains the original map elements plus mapping for the components.
 // This is because when a type is 'owned' by the template
 // instantiator, all parts of the type are owned.  We only consider
-// type-components as typed.
+// type-components as written.
 static map<const Type*, const Type*> ResugarTypeComponents(
     const map<const Type*, const Type*>& resugar_map) {
   map<const Type*, const Type*> retval = resugar_map;
-  for (Each<const Type*, const Type*> it(&resugar_map); !it.AtEnd(); ++it) {
-    const set<const Type*>& components = GetComponentsOfType(it->second);
-    for (Each<const Type*> component_type(&components);
-         !component_type.AtEnd(); ++component_type) {
-      const Type* desugared_type = GetCanonicalType(*component_type);
+  for (const auto& types : resugar_map) {
+    const set<const Type*>& components = GetComponentsOfType(types.second);
+    for (const Type* component_type : components) {
+      const Type* desugared_type = GetCanonicalType(component_type);
       if (!ContainsKey(retval, desugared_type)) {
-        retval[desugared_type] = *component_type;
+        retval[desugared_type] = component_type;
         VERRS(6) << "Adding a type-components of interest: "
-                 << PrintableType(*component_type) << "\n";
+                 << PrintableType(component_type) << "\n";
       }
     }
   }
@@ -697,7 +689,7 @@ static map<const Type*, const Type*> ResugarTypeComponents(
 static map<const Type*, const Type*> GetTplTypeResugarMapForFunctionNoCallExpr(
     const FunctionDecl* decl, unsigned start_arg) {
   map<const Type*, const Type*> retval;
-  if (!decl)   // can be NULL if the function call is via a function pointer
+  if (!decl)   // can be nullptr if the function call is via a function pointer
     return retval;
   if (const TemplateArgumentList* tpl_list
       = decl->getTemplateSpecializationArgs()) {
@@ -731,12 +723,12 @@ map<const Type*, const Type*> GetTplTypeResugarMapForFunction(
     const FunctionDecl* decl, const Expr* calling_expr) {
   map<const Type*, const Type*> retval;
 
-  // If calling_expr is NULL, then we can't find any explicit template
+  // If calling_expr is nullptr, then we can't find any explicit template
   // arguments, if they were specified (e.g. 'Fn<int>()'), and we
   // won't be able to get the function arguments as written.  So we
   // can't resugar at all.  We just have to hope that the types happen
   // to be already sugared, because the actual-type is already canonical.
-  if (calling_expr == NULL) {
+  if (calling_expr == nullptr) {
     retval = GetTplTypeResugarMapForFunctionNoCallExpr(decl, 0);
     retval = ResugarTypeComponents(retval);  // add in retval's decomposition
     return retval;
@@ -748,7 +740,7 @@ map<const Type*, const Type*> GetTplTypeResugarMapForFunction(
   // arguments might be explicit, and others implicit.  Otherwise,
   // it's a type that doesn't take function template args at all (like
   // CXXDeleteExpr) or only takes explicit args (like DeclRefExpr).
-  const Expr* const* fn_args = NULL;
+  const Expr* const* fn_args = nullptr;
   unsigned num_args = 0;
   unsigned start_of_implicit_args = 0;
   if (const CXXConstructExpr* ctor_expr = DynCastFrom(calling_expr)) {
@@ -801,24 +793,24 @@ map<const Type*, const Type*> GetTplTypeResugarMapForFunction(
     InsertAllInto(GetComponentsOfType(argtype), &fn_arg_types);
   }
 
-  for (Each<const Type*> it(&fn_arg_types); !it.AtEnd(); ++it) {
+  for (const Type* type : fn_arg_types) {
     // See if any of the template args in retval are the desugared form of us.
-    const Type* desugared_type = GetCanonicalType(*it);
+    const Type* desugared_type = GetCanonicalType(type);
     if (ContainsKey(desugared_types, desugared_type)) {
-      retval[desugared_type] = *it;
-      if (desugared_type != *it) {
+      retval[desugared_type] = type;
+      if (desugared_type != type) {
         VERRS(6) << "Remapping template arg of interest: "
                  << PrintableType(desugared_type) << " -> "
-                 << PrintableType(*it) << "\n";
+                 << PrintableType(type) << "\n";
       }
     }
   }
 
   // Log the types we never mapped.
-  for (Each<const Type*, const Type*> it(&desugared_types); !it.AtEnd(); ++it) {
-    if (!ContainsKey(retval, it->first)) {
+  for (const auto& types : desugared_types) {
+    if (!ContainsKey(retval, types.first)) {
       VERRS(6) << "Ignoring unseen-in-fn-args template arg of interest: "
-               << PrintableType(it->first) << "\n";
+               << PrintableType(types.first) << "\n";
     }
   }
 
@@ -904,10 +896,15 @@ bool IsFriendDecl(const Decl* decl) {
   return decl->getFriendObjectKind() != Decl::FOK_None;
 }
 
-bool IsForwardDecl(const clang::TagDecl* decl) {
-  return (isa<RecordDecl>(decl) &&   // not an enum
-          !decl->isCompleteDefinition() && !IsFriendDecl(decl) &&
-          !decl->isEmbeddedInDeclarator());
+bool IsForwardDecl(const NamedDecl* decl) {
+  if (const auto* record_decl = dyn_cast<RecordDecl>(decl)) {
+    return (!record_decl->getName().empty() &&
+            !record_decl->isCompleteDefinition() &&
+            !record_decl->isEmbeddedInDeclarator() &&
+            !IsFriendDecl(record_decl));
+  }
+
+  return false;
 }
 
 // Two possibilities: it's written as a nested class (that is, with a
@@ -990,12 +987,12 @@ const NamedDecl* GetFirstRedecl(const NamedDecl* decl) {
   FullSourceLoc first_decl_loc(GetLocation(first_decl), *GlobalSourceManager());
   set<const NamedDecl*> all_redecls = GetClassRedecls(decl);
   if (all_redecls.empty())  // input is not a class or class template
-    return NULL;
+    return nullptr;
 
-  for (Each<const NamedDecl*> it(&all_redecls); !it.AtEnd(); ++it) {
-    const FullSourceLoc redecl_loc(GetLocation(*it), *GlobalSourceManager());
+  for (const NamedDecl* redecl : all_redecls) {
+    const FullSourceLoc redecl_loc(GetLocation(redecl), *GlobalSourceManager());
     if (redecl_loc.isBeforeInTranslationUnitThan(first_decl_loc)) {
-      first_decl = *it;
+      first_decl = redecl;
       first_decl_loc = redecl_loc;
     }
   }
@@ -1024,7 +1021,6 @@ bool DeclsAreInSameClass(const Decl* decl1, const Decl* decl2) {
   return decl1->getDeclContext()->isRecord();
 }
 
-
 // --- Utilities for Type.
 
 const Type* GetTypeOf(const Expr* expr) {
@@ -1086,7 +1082,7 @@ const Type* RemovePointerFromType(const Type* type) {
     type = type->getUnqualifiedDesugaredType();
   }
   if (!IsPointerOrReferenceAsWritten(type)) {
-    return NULL;
+    return nullptr;
   }
   type = RemoveElaboration(type);
   type = type->getPointeeType().getTypePtr();
@@ -1095,10 +1091,10 @@ const Type* RemovePointerFromType(const Type* type) {
 
 // This follows typedefs/etc to remove pointers, if necessary.
 const Type* RemovePointersAndReferences(const Type* type) {
-  while (1) {
+  while (true) {
     const Type* deref_type = RemovePointerFromType(type);
-    if (deref_type == NULL)   // type wasn't a pointer (or reference) type
-      break;                  // removed all pointers
+    if (deref_type == nullptr) // type wasn't a pointer (or reference) type
+      break;                   // removed all pointers
     type = deref_type;
   }
   return type;
@@ -1136,9 +1132,9 @@ const NamedDecl* TypeToDeclAsWritten(const Type* type) {
   } else if (const FunctionType* function_type = DynCastFrom(type)) {
     // TODO(csilvers): is it possible to map from fn type to fn decl?
     (void)function_type;
-    return NULL;
+    return nullptr;
   }  else {
-    return NULL;
+    return nullptr;
   }
 }
 
@@ -1198,13 +1194,13 @@ GetTplTypeResugarMapForClassNoComponentTypes(const clang::Type* type) {
     // and compare it against each of the types in the template decl
     // (the latter are all desugared).  If there's a match, update
     // the mapping.
-    for (Each<const Type*> it(&arg_components); !it.AtEnd(); ++it) {
+    for (const Type* type : arg_components) {
       for (unsigned i = 0; i < tpl_args.size(); ++i) {
         if (const Type* arg_type = GetTemplateArgAsType(tpl_args[i])) {
-          if (GetCanonicalType(*it) == arg_type) {
-            retval[arg_type] = *it;
+          if (GetCanonicalType(type) == arg_type) {
+            retval[arg_type] = type;
             VERRS(6) << "Adding a template-class type of interest: "
-                     << PrintableType(*it) << "\n";
+                     << PrintableType(type) << "\n";
             explicit_args.insert(i);
           }
         }
@@ -1217,7 +1213,7 @@ GetTplTypeResugarMapForClassNoComponentTypes(const clang::Type* type) {
     if (ContainsKey(explicit_args, i))
       continue;
     if (const Type* arg_type = GetTemplateArgAsType(tpl_args[i])) {
-      retval[arg_type] = NULL;
+      retval[arg_type] = nullptr;
       VERRS(6) << "Adding a template-class default type of interest: "
                << PrintableType(arg_type) << "\n";
     }
@@ -1253,7 +1249,7 @@ const Type* TypeOfParentIfMethod(const CallExpr* expr) {
       return ref_expr->getQualifier()->getAsType();
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const Expr* GetFirstClassArgument(CallExpr* expr) {
@@ -1273,23 +1269,23 @@ const Expr* GetFirstClassArgument(CallExpr* expr) {
       return *it;
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const CXXDestructorDecl* GetDestructorForDeleteExpr(const CXXDeleteExpr* expr) {
   const Type* type = expr->getDestroyedType().getTypePtrOrNull();
-  // type is NULL when deleting a dependent type: 'T foo; delete foo'
-  if (type == NULL)
-    return NULL;
+  // type is nullptr when deleting a dependent type: 'T foo; delete foo'
+  if (type == nullptr)
+    return nullptr;
   const NamedDecl* decl = TypeToDeclAsWritten(type);
   if (const CXXRecordDecl* cxx_record = DynCastFrom(decl))
     return cxx_record->getDestructor();
-  return NULL;
+  return nullptr;
 }
 
 const CXXDestructorDecl* GetSiblingDestructorFor(
     const CXXConstructorDecl* ctor) {
-  return ctor ? ctor->getParent()->getDestructor() : NULL;
+  return ctor ? ctor->getParent()->getDestructor() : nullptr;
 }
 
 const CXXDestructorDecl* GetSiblingDestructorFor(
