@@ -34,7 +34,6 @@ using clang::SourceLocation;
 using clang::UnaryOperator;
 using clang::UnresolvedMemberExpr;
 
-
 namespace include_what_you_use {
 
 // This works around two bugs(?) in clang where decl->getLocation()
@@ -63,7 +62,7 @@ namespace include_what_you_use {
 //    Note the two issues can both be present, if an implicit method's
 // parent is an implicit instantiation.
 SourceLocation GetLocation(const clang::Decl* decl) {
-  if (decl == NULL)  return SourceLocation();
+  if (decl == nullptr)  return SourceLocation();
 
   if (const CXXMethodDecl* method_decl = DynCastFrom(decl)) {
     if (method_decl->isImplicit())
@@ -119,7 +118,7 @@ static SourceLocation GetMemberExprLocation(const MemberExpr* member_expr) {
 }
 
 SourceLocation GetLocation(const clang::Stmt* stmt) {
-  if (stmt == NULL)  return SourceLocation();
+  if (stmt == nullptr)  return SourceLocation();
   // For some expressions, we take the location to be the 'key' part
   // of the expression, not the beginning.  For instance, the
   // location of 'a << b' is the '<<', not the 'a'.  This is
@@ -153,17 +152,17 @@ SourceLocation GetLocation(const clang::Stmt* stmt) {
 }
 
 SourceLocation GetLocation(const clang::TypeLoc* typeloc) {
-  if (typeloc == NULL)  return SourceLocation();
+  if (typeloc == nullptr)  return SourceLocation();
   return typeloc->getBeginLoc();
 }
 
 SourceLocation GetLocation(const clang::NestedNameSpecifierLoc* nnsloc) {
-  if (nnsloc == NULL)  return SourceLocation();
+  if (nnsloc == nullptr)  return SourceLocation();
   return nnsloc->getBeginLoc();
 }
 
 SourceLocation GetLocation(const clang::TemplateArgumentLoc* argloc) {
-  if (argloc == NULL)  return SourceLocation();
+  if (argloc == nullptr)  return SourceLocation();
   return argloc->getLocation();
 }
 
